<?php

namespace Drupal\views_csv_source\Plugin\views\filter;

use Drupal\Core\Form\FormStateInterface;
use Drupal\views\Plugin\views\display\DisplayPluginBase;
use Drupal\views\Plugin\views\filter\InOperator;
use Drupal\views\ViewExecutable;
use Drupal\views_csv_source\Plugin\views\ColumnSelectorTrait;
use Drupal\views_csv_source\Plugin\views\query\ViewsCsvQuery;

/**
 * Filter handler for views_csv_source with options.
 *
 * @ingroup views_filter_handlers
 *
 * @ViewsFilter("views_csv_source_filter_select")
 */
class ViewsCsvFilterSelect extends InOperator {

  use ColumnSelectorTrait;
  use ViewsCsvFilterTrait;

  /**
   * {@inheritdoc}
   */
  public function buildOptionsForm(&$form, FormStateInterface $form_state): void {
    parent::buildOptionsForm($form, $form_state);
    $form = $this->buildKeyOptionElement($form);

    // Adding ajax behavior on the column key.
    $form['column_key_button'] = [
      '#prefix' => '<div class="views-csv-source clearfix">',
      '#suffix' => '</div>',
      // Should always come after the expose button.
      '#weight' => -199,
    ];
    $form['column_key_button']['key'] = [
      '#theme_wrappers' => ['container'],
      '#attributes' => ['class' => ['js-only']],
    ];
    $form['column_key_button']['key']['key'] = $form['key'];
    unset($form['key']);
    $form['column_key_button']['key']['key']['#attached']['library'][] = 'views_csv_source/views-admin';
    $form['column_key_button']['key']['key']['#attributes']['data-views-csv-source-filter-key-selector'] = TRUE;

    // Refresh the value input based on the selected column using ajax.
    $form['column_key_button']['button'] = [
      '#limit_validation_errors' => [],
      '#type' => 'submit',
      '#value' => $this->t('Refresh value'),
      '#submit' => [[$this, 'refreshValueInput']],
    ];
  }

  /**
   * Submit handler to refresh the value options when the column key change.
   */
  public function refreshValueInput($form, FormStateInterface $form_state): void {
    $item = &$this->options;
    $view = $form_state->get('view');
    $display_id = $form_state->get('display_id');
    $type = $form_state->get('type');
    $id = $form_state->get('id');

    // If the user made a choice in the key field, then we need to build a list
    // of options from the selected column in the CSV file.
    $inputs = $form_state->getUserInput();
    $selected_column_key = $inputs['options']['column_key_button']['key']['key'] ?? '';
    if ($selected_column_key !== $item['key']) {
      $item['key'] = $selected_column_key;
      $item['value'] = [];
    }
    $view->getExecutable()->setHandler($display_id, $type, $id, $item);

    $view->addFormToStack($form_state->get('form_key'), $display_id, $type, $id, TRUE, TRUE);

    $form_state->set('rerender', TRUE);
    $form_state->setRebuild();
    // Write to cache.
    $view->cacheSet();
  }

  /**
   * {@inheritdoc}
   */
  public function init(ViewExecutable $view, DisplayPluginBase $display, ?array &$options = NULL) {
    parent::init($view, $display, $options);
    $this->valueTitle = $this->t('Value');
    $this->definition['options callback'] = [$this, 'getColumnOptions'];

    $query = $this->view->getQuery();
    // Initialize the key to the first column header if empty.
    if ($this->options['key'] === '' && $headers = $query->getCsvHeader()) {
      $this->options['key'] = reset($headers);
    }
    // Pass the selected key to the options' getter argument.
    $this->definition['options arguments'] = [$this->options['key']];
  }

  /**
   * {@inheritdoc}
   */
  public function query(): void {
    $info = $this->operators();
    if (!empty($info[$this->operator]['method'])) {
      $this->{$info[$this->operator]['method']}($this->options['key']);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function validate() {
    if (!empty($this->value)) {
      parent::validate();
    }
  }

  /**
   * {@inheritdoc}
   */
  protected function opSimple(string $column_name = ''): void {
    if (empty($this->value) || !$column_name) {
      return;
    }

    $this->query->addWhere($this->options['group'], $column_name, array_values($this->value), $this->operator);
  }

  /**
   * {@inheritdoc}
   */
  protected function opEmpty(string $column_name = '') {
    if (!$column_name) {
      return;
    }

    $operator = $this->operator == 'empty' ? "IS NULL" : "IS NOT NULL";
    $this->query->addWhere($this->options['group'], $column_name, NULL, $operator);
  }

  /**
   * Get the column options.
   *
   * @param string $column
   *   The column name.
   *
   * @return array
   *   The column options.
   */
  protected function getColumnOptions(string $column): array {
    if (!$column) {
      return [];
    }

    $query = $this->view->getQuery();
    // Ensure the query is of the correct type.
    if ($query instanceof ViewsCsvQuery) {
      $column_data = $query->getCsvColumnValues($column);
      return array_combine($column_data, $column_data);
    }

    return [];
  }

}
