<?php

namespace Drupal\Tests\views_csv_source\Unit\Query;

use Drupal\Core\Database\Query\Condition;
use Drupal\Core\Database\Query\ConditionInterface;
use Drupal\Tests\UnitTestCase;
use Drupal\views_csv_source\Query\Connection;
use Drupal\views_csv_source\Query\Select;
use GuzzleHttp\Client as HttpClient;
use GuzzleHttp\Handler\MockHandler;
use GuzzleHttp\HandlerStack;
use GuzzleHttp\Psr7\Response;

/**
 * @coversDefaultClass \Drupal\views_csv_source\Query\Select
 *
 * @group views_csv_source
 */
class SelectTest extends UnitTestCase {

  /**
   * The connection object.
   *
   * @var \Drupal\views_csv_source\Query\Connection
   */
  protected $connection;

  /**
   * The URI of the CSV file.
   *
   * @var string
   */
  protected $csvUri = '';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->csvUri = $this->resourcesPath() . '/views_csv_data_test.csv';

    $this->connection = new Connection(
      $this->createMock('Drupal\Core\Cache\CacheBackendInterface'),
      $this->createHttpClientMock(),
      $this->createMock('Drupal\Component\Datetime\TimeInterface'),
      $this->createMock('Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher'),
      $this->getConfigFactoryStub(['views_csv_source.settings' => ['cache_ttl' => 3600]]),
      $this->createMock('Drupal\Core\Logger\LoggerChannelFactoryInterface'),
    );

  }

  /**
   * @covers ::__construct
   * @covers ::execute
   * @covers ::getRecords
   */
  public function testSimpleExecute() {
    $select = $this->getSelectQuery();
    $this->assertInstanceOf(Select::class, $select);
    $records = $select->execute();
    // Expecting an array since we didn't add any fields.
    $this->assertIsIterable($records);
    // Expecting zero here since we didn't add any fields.
    $this->assertCount(0, $records);
  }

  /**
   * @covers ::addField
   * @covers ::addColumn
   * @covers ::execute
   * @covers ::getRecords
   */
  public function testExecuteWithFields() {
    $select = $this->getSelectQuery()
      ->addField('Geography name')
      ->addField('Occupation Name');
    $records = $select->execute();

    $records = array_values(iterator_to_array($records));

    $this->assertCount(7, $records);

    $this->assertArrayHasKey('Geography name', $records[0]);
    $this->assertArrayHasKey('Occupation Name', $records[0]);
    $this->assertArrayNotHasKey('Total count', $records[0]);

    $this->assertEquals([
      'Geography name' => 'Hawaii',
      'Occupation Name' => 'Health Practitioner Support Technologists and Technicians',
    ], $records[4]);
  }

  /**
   * @covers ::addField
   * @covers ::addColumn
   * @covers ::condition
   * @covers ::execute
   * @covers ::applyFilters
   *
   * @dataProvider conditionDataProvider
   */
  public function testExecuteWithFieldAndCondition(ConditionInterface $condition, string $field, array $expected_first_record, int $expected_count) {
    $select = $this->getSelectQuery()
      ->addField($field)
      ->condition($condition);

    $records = $select->execute();
    $records = array_values(iterator_to_array($records));

    $this->assertCount($expected_count, $records);
    $this->assertEquals($expected_first_record, $records[0]);
  }

  /**
   * @covers ::condition
   * @covers ::execute
   * @covers ::applyFilters
   * @covers ::getRecords
   * @covers ::getSelectedFields
   * @covers ::buildConditions
   * @covers ::conditionPassed
   * @covers ::verifyCondition
   */
  public function testExecuteWithFieldAndNestedConditions() {
    $condition = new Condition('AND');
    $condition->condition('Geography name', 'Hawaii');
    $condition->condition('Occupation Name', 'Health Practitioner Support Technologists and Technicians');

    $select = $this->getSelectQuery()
      ->addField('Geography name')
      ->condition($condition);

    $records = $select->execute();
    $records = array_values(iterator_to_array($records));

    $this->assertCount(1, $records);
    $this->assertEquals([
      'Geography name' => 'Hawaii',
    ], $records[0]);
  }

  /**
   * @covers ::groupBy
   * @covers ::execute
   * @covers ::applyGroupBy
   * @covers ::applyFilters
   * @covers ::getRecords
   * @covers ::extractGroupedColumnsValuesFromRecord
   * @covers ::processGroupByAndAggregation
   */
  public function testGroupBy() {
    $select = $this->getSelectQuery()
      ->addField('Geography name', 'group_by')
      ->addField('Total count', 'sum')
      ->groupBy('Geography name');

    $records = $select->execute();
    $records = array_values(iterator_to_array($records));

    $this->assertCount(5, $records);

    // Ensure that the first record is Alabama and the Total count is 101.
    $this->assertEquals([
      'Geography name' => 'Alabama',
      'Total count' => '101',
    ], $records[0]);
  }

  /**
   * @covers ::groupBy
   * @covers ::orderBy
   * @covers ::execute
   * @covers ::applyGroupBy
   * @covers ::applyFilters
   * @covers ::getRecords
   * @covers ::extractGroupedColumnsValuesFromRecord
   * @covers ::processGroupByAndAggregation
   * @covers ::buildCsvReader
   * @covers ::buildNewCsvFromRecords
   * @covers ::applyOrderBy
   */
  public function testGroupByWithOrderBy() {
    $select = $this->getSelectQuery()
      ->addField('Geography name', 'group_by')
      ->addField('Total count', 'sum')
      ->groupBy('Geography name')
      ->orderBy('Total count', Select::DIRECTION_DESC);

    $records = $select->execute();
    $records = array_values(iterator_to_array($records));

    $this->assertCount(5, $records);

    // Ensure that the first record is Alaska and the Total count is 9045.
    $this->assertEquals([
      'Geography name' => 'Alaska',
      'Total count' => '9045',
    ], $records[0]);
  }

  /**
   * @covers ::range
   */
  public function testGroupByWithOrderByAndRange() {
    $select = $this->getSelectQuery()
      ->addField('Geography name', 'group_by')
      ->addField('Total count', 'sum')
      ->groupBy('Geography name')
      ->orderBy('Total count', Select::DIRECTION_DESC)
      ->range(0, 2);

    $records = $select->execute();
    $records = array_values(iterator_to_array($records));

    $this->assertCount(2, $records);
    $this->assertEquals([
      [
        'Geography name' => 'Alaska',
        'Total count' => '9045',
      ],
      [
        'Geography name' => 'Hawaii',
        'Total count' => '543',
      ],
    ], $records);
  }

  /**
   * @covers ::range
   * @covers ::setLimit
   * @covers ::setOffset
   * @covers ::getLimit
   * @covers ::getOffset
   */
  public function testGroupByWithOrderByAndLimitAndOffset() {
    $select = $this->getSelectQuery()
      ->addField('Geography name', 'group_by')
      ->addField('Total count', 'sum')
      ->groupBy('Geography name')
      ->orderBy('Total count', Select::DIRECTION_DESC)
      ->setLimit(2)
      ->setOffset(1);

    $this->assertEquals(2, $select->getLimit());
    $this->assertEquals(1, $select->getOffset());

    $records = $select->execute();
    $records = array_values(iterator_to_array($records));

    $this->assertCount(2, $records);

    $this->assertEquals([
      [
        'Geography name' => 'Hawaii',
        'Total count' => '543',
      ],
      [
        'Geography name' => 'Alabama',
        'Total count' => '101',
      ],
    ], $records);
  }

  /**
   * Data provider for ::testExecuteWithFieldAndCondition().
   */
  public function conditionDataProvider(): array {
    $conditions = [];

    // Simple equal operator.
    $condition = new Condition('AND');
    $condition->condition('Geography name', 'Hawaii');
    $conditions['simple_equal_operator'] = [
      $condition,
      'Geography name',
      ['Geography name' => 'Hawaii'],
      1,
    ];

    // Contains operator test.
    $condition = new Condition('AND');
    $condition->condition('Occupation Name', 'Technicians', 'contains');
    $conditions['contains_operator'] = [
      $condition,
      'Occupation Name',
      ['Occupation Name' => 'Emergency Medical Technicians and Paramedics'],
      3,
    ];
    return $conditions;
  }

  /**
   * Gets the select query object.
   *
   * @return \Drupal\views_csv_source\Query\Select
   *   The select query object.
   */
  protected function getSelectQuery(): Select {
    return $this->connection->select($this->csvUri, [
      'headers' => ['Content-Type' => 'application/csv'],
      'cache_id' => 'views_csv_source_data_test',
      'csv_file' => $this->csvUri,
    ]);
  }

  /**
   * Gets the absolute path.
   *
   * @return string
   *   The absolute path.
   */
  protected function absolutePath(): string {
    return dirname(__DIR__, 4);
  }

  /**
   * Gets the resources path.
   *
   * @return string
   *   The resources path.
   */
  protected function resourcesPath(): string {
    return $this->absolutePath() . '/tests/resources';
  }

  /**
   * Returns a mock object for the httpClient.
   *
   * @return \GuzzleHttp\Client
   *   The mocked object,
   */
  protected function createHttpClientMock(): HttpClient {
    // Mocking the Guzzle response to return the test CSV file content.
    $body = file_get_contents($this->csvUri);
    $mock = new MockHandler([
      new Response(200, [], $body),
    ]);
    $handler = HandlerStack::create($mock);
    return new HttpClient(['handler' => $handler]);
  }

}
